/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 2.0
 */

using System;
using System.Collections;
using System.Web;
using System.Web.UI;

namespace TopCoder.Web.UI.WebControl.BreadCrumb
{
    /// <summary>
    /// This abstract implementation of the <see cref="IPathWriter"/> will provide some of the common functionality
    /// when writing a path.
    /// </summary>
    /// <remarks>
    /// <para>
    /// This abstract implementation will provide the following functionality:
    /// <list>
    /// <item>Provides three methods: <see cref="BeginPath"/>, <see cref="WriteNode"/> and <see cref="EndPath"/> that
    /// will allow implementations to concentrate on the rendering and not the processing of the path.</item>
    /// <item>Processes the node list in the way the associated <see cref="BreadCrumbTrailControl"/> wants (i.e. the
    /// <see cref="PathDirection"/>) and will call the various methods with the appropriate parameters (i.e. root node
    /// style on the root, current node style on the current node and node style on all others</item>
    /// <item>The separator (if specified) will be written inbetween the nodes automatically.</item>
    /// </list>
    /// </para>
    /// <para>
    /// It's recommended that all implementations of <see cref="IPathWriter"/> inherit from this abstract base to not
    /// duplicate the processing of the path and to 'future' proof the implementations against future negligible
    /// changes. This abstract base is thread safe by being immutable and only processing on the rendering thread.
    /// </para>
    /// </remarks>
    public abstract class AbstractPathWriter : IPathWriter
    {
        /// <summary>
        /// Constructor performing any needed initialization. Should be called by subclasses.
        /// </summary>
        protected AbstractPathWriter()
        {
        }

        /// <summary>
        /// Writes a path by iterating over the nodes, between calls to <see cref="BeginPath"/> and
        /// <see cref="EndPath"/>. <see cref="WriteNode"/> and <see cref="WriteSeparator"/> are called accordingly.
        /// Necessary steps are taken to maintain correct exception handling. <seealso cref="ErrorDisposition"/>
        /// </summary>
        /// <param name="writer">A non-null writer to use.</param>
        /// <param name="nodes">A non-null, non-empty array of nodes to render.</param>
        /// <param name="parent">A non-null parent control to use.</param>
        /// <exception cref="ArgumentNullException">If <paramref name="writer"/>, <paramref name="nodes"/> or
        /// <paramref name="parent"/> is null.</exception>
        /// <exception cref="ArgumentException">If <paramref name="nodes"/> is empty, or any element is not initialized
        /// (null URL).
        /// </exception>
        /// <returns>null if no exceptions occur or a >= 1 length array of exceptions that occurred.</returns>
        public virtual BreadCrumbException[] WritePath(HtmlTextWriter writer, BreadCrumbNode[] nodes,
            BreadCrumbTrailControl parent)
        {
            if (writer == null)
            {
                throw new ArgumentNullException("writer");
            }
            if (nodes == null)
            {
                throw new ArgumentNullException("nodes");
            }
            if (parent == null)
            {
                throw new ArgumentNullException("parent");
            }

            if (nodes.Length == 0)
            {
                throw new ArgumentException("Empty array specified.", "nodes");
            }
            foreach (BreadCrumbNode node in nodes)
            {
                if (node.Url == null)
                {
                    throw new ArgumentException("Non-inited BreadCrumbNode present.", "nodes");
                }
            }

            ArrayList exceptionList = new ArrayList();

            try
            {
                // Even if we can't init, it's possible that we can go on. That really depends on the specific writer,
                // and we don't know anything about that.
                try
                {
                    BeginPath(writer);
                }
                catch (Exception e)
                {
                    HandleException(exceptionList, e, parent.ErrorDisposition);
                }

                // These are the values for the "root to child" direction
                int offset = 0;
                int multiple = 1;

                if (parent.PathDirection == PathDirection.CurrentToRoot)
                {
                    offset = nodes.Length - 1;
                    multiple = -1;
                }

                for (int i = 0; i < nodes.Length; i++)
                {
                    // Write a separator if we're not writing the first trail part, note that this is dependent on our
                    // iteration index, not the array index
                    if (i != 0)
                    {
                        try
                        {
                            WriteSeparator(writer, parent);
                        }
                        catch (Exception e)
                        {
                            HandleException(exceptionList, e, parent.ErrorDisposition);
                        }
                    }
                    try
                    {
                        // Get the right index
                        int index = offset + i * multiple;

                        // The default style
                        string style = parent.NodeStyle;

                        // Root > current > ordinary node, always "fall back" if one is null
                        if (index == nodes.Length - 1 && parent.CurrentNodeStyle != null)
                        {
                            style = parent.CurrentNodeStyle;
                        }
                        if (index == 0 && parent.RootNodeStyle != null)
                        {
                            style = parent.RootNodeStyle;
                        }

                        WriteNode(writer, nodes[index], style, parent.MouseOverStyle);
                    }
                    catch (Exception e)
                    {
                        HandleException(exceptionList, e, parent.ErrorDisposition);
                    }
                }

                // An exception here means nothing
                EndPath(writer);
            }
            catch (Exception e)
            {
                BreadCrumbException bce = WrapIfNeeded(e);

                switch (parent.ErrorDisposition)
                {
                    case ErrorDisposition.RenderException:
                        exceptionList.Add(bce);
                        break;
                    case ErrorDisposition.SilentlyIgnore:
                        break;
                    case ErrorDisposition.ThrowException:
                        return new BreadCrumbException[1] { bce };
                }
            }

            if (exceptionList.Count > 0)
            {
                BreadCrumbException[] list = new BreadCrumbException[exceptionList.Count];
                exceptionList.CopyTo(list);

                return list;
            }

            // No exceptions
            return null;
        }

        /// <summary>
        /// This method is called prior to processing the node path and will allow subimplementations override to
        /// provide initialization logic. This base version does nothing, except for verifying the parameter being
        /// non-null.
        /// </summary>
        /// <param name="writer">A non-null HTML writer.</param>
        /// <exception cref="ArgumentNullException">If <paramref name="writer"/> is null.</exception>
        /// <exception cref="BreadCrumbException">Any error.</exception>
        protected virtual void BeginPath(HtmlTextWriter writer)
        {
            if (writer == null)
            {
                throw new ArgumentNullException("writer");
            }
        }

        /// <summary>
        /// Handles an exception according to an error disposition, under the assumption that there is a final, outer
        /// handler.
        /// </summary>
        /// <param name="exceptionList">The exception list, that is, a list of zero or more already encountered
        /// exceptions. We might add to this list.</param>
        /// <param name="e">The exception itself.</param>
        /// <param name="disposition">The setting defining our error handling.</param>
        private static void HandleException(IList exceptionList, Exception e, ErrorDisposition disposition)
        {
            switch (disposition)
            {
                case ErrorDisposition.SilentlyIgnore:
                    break;
                case ErrorDisposition.RenderException:
                    exceptionList.Add(WrapIfNeeded(e));
                    break;
                case ErrorDisposition.ThrowException:
                    // Throw the exception here, we don't want to go on running, the outer handler will adapt this.
                    throw e;
            }
        }

        /// <summary>
        /// A simple helper that wraps an exception if it's not already a <see cref="BreadCrumbException"/>.
        /// </summary>
        /// <param name="e">The exception to (possibly) wrap.</param>
        /// <returns>The resulting exception.</returns>
        private static BreadCrumbException WrapIfNeeded(Exception e)
        {
            if (e is BreadCrumbException)
            {
                return (BreadCrumbException) e;
            }

            return new BreadCrumbException("Error encountered while writing path.", e);
        }

        /// <summary>
        /// This method will be called once for each node in the path with the appropriate node style. Subclasses will
        /// implement this abstract method to provide the processing for each node.
        /// </summary>
        /// <param name="writer">A non-null writer to use.</param>
        /// <param name="node">A non-null node to write.</param>
        /// <param name="nodeStyle">A style class to use for the node or null if no style should be used.</param>
        /// <param name="mouseOverStyle">The mouse over style class to use or null for none.</param>
        /// <exception cref="ArgumentNullException">If <paramref name="writer"/> is null, or the URL of
        /// <paramref name="node"/> is null.
        /// </exception>
        /// <exception cref="BreadCrumbException">Any error.</exception>
        protected abstract void WriteNode(HtmlTextWriter writer, BreadCrumbNode node, string nodeStyle,
            string mouseOverStyle);

        /// <summary>
        /// Writes the separator if specified.
        /// </summary>
        /// <remarks>
        /// If <paramref name="parent"/> has a specified separator, it's written. If there is a path separator style,
        /// that style is used. The separator string is escaped/encoded for HTML when written.
        /// </remarks>
        /// <param name="writer">A non-null writer to use.</param>
        /// <param name="parent">A non-null parent control to use.</param>
        /// <exception cref="ArgumentNullException">If <paramref name="writer"/> or <paramref name="parent"/> is null.
        /// </exception>
        /// <exception cref="BreadCrumbException">Any error.</exception>
        protected virtual void WriteSeparator(HtmlTextWriter writer, BreadCrumbTrailControl parent)
        {
            if (writer == null)
            {
                throw new ArgumentNullException("writer");
            }
            if (parent == null)
            {
                throw new ArgumentNullException("parent");
            }

            try
            {
                // Return if there is no path separator
                if (parent.PathSeparator == null)
                {
                    return;
                }

                // Add a path separator style class specification, if desired
                if (parent.PathSeparatorStyle != null)
                {
                    writer.AddAttribute(HtmlTextWriterAttribute.Class, parent.PathSeparatorStyle);
                }

                // Write the separator, escaped and enclosed in a <span> element
                writer.RenderBeginTag(HtmlTextWriterTag.Span);
                writer.Write(HttpUtility.HtmlEncode(parent.PathSeparator));
                writer.RenderEndTag();
            }
            catch (Exception e)
            {
                throw new BreadCrumbException("Error writing separator.", e);
            }
        }

        /// <summary>
        /// This method is called post processing the node path and will allow subimplementations override to provide
        /// cleanup logic. This base version does nothing, except for verifying the parameter being non-null.
        /// </summary>
        /// <param name="writer">A non-null writer to use.</param>
        /// <exception cref="ArgumentNullException">If <paramref name="writer"/> is null.</exception>
        /// <exception cref="BreadCrumbException">Any error.</exception>
        protected virtual void EndPath(HtmlTextWriter writer)
        {
            if (writer == null)
            {
                throw new ArgumentNullException("writer");
            }
        }
    }
}
