/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 2.0
 */

using System;

namespace TopCoder.Web.UI.WebControl.BreadCrumb
{
    /// <summary>
    /// A structure for holding URL and the page title of node.
    /// </summary>
    /// <remarks>
    /// This structure will be created by the <see cref="INodeDiscovery"/> (when discovering the current node) and by
    /// the <see cref="IPathDiscovery"/> implementations to return the path from the current node to the root node.
    /// Each node in that path will be represented by this class. This class is immutable and thread safe.
    /// </remarks>
    [Serializable]
    public struct BreadCrumbNode
    {
        /// <summary>
        /// Represents the title associated with this node. This variable is set by the constructor and is immutable.
        /// This value of this can be null if the title should default to the title found in the
        /// <see cref="IPathDiscovery"/>. This value should not be null if it's a node that is part of a path returned
        /// by the <see cref="IPathDiscovery"/> implementation. This variable is referenced by the associated getter.
        /// </summary>
        private readonly string title;

        /// <summary>
        /// Represents the title associated with this node. The value is set by the constructor and is immutable.
        /// This value of this can be null if the title should default to the title found in the
        /// <see cref="IPathDiscovery"/>. This value should not be null if it's a node that is part of a path returned
        /// by the <see cref="IPathDiscovery"/> implementation.
        /// </summary>
        /// <value>
        /// A null or non-empty string.
        /// </value>
        public string Title
        {
            get
            {
                return title;
            }
        }

        /// <summary>
        /// Represents the URL (including sorted query strings if any) associated with this node. This variable is set
        /// by the constructor, is immutable and will never be null. This variable is referenced by the associated
        /// getter.
        /// </summary>
        private readonly string url;

        /// <summary>
        /// Represents the URL (including sorted query strings if any) associated with this node. This value is set
        /// by the constructor, is immutable and will never be null. This variable is referenced by the associated
        /// getter.
        /// </summary>
        /// <value>
        /// A non-null, non-empty string.
        /// </value>
        public string Url
        {
            get
            {
                return url;
            }
        }

        /// <summary>
        /// Constructor for <see cref="BreadCrumbNode"/> from the specified variables.
        /// </summary>
        /// <param name="title">A string, possibly null, possibly empty. Empty is converted to null.</param>
        /// <param name="url">A non-null, non-empty string.</param>
        /// <exception cref="ArgumentNullException">If <paramref name="url"/> is null.</exception>
        /// <exception cref="ArgumentException">If <paramref name="url"/> is empty (all whitespace).</exception>
        public BreadCrumbNode(string title, string url)
        {
            if (url == null)
            {
                throw new ArgumentNullException("url");
            }
            if (url.Trim().Length == 0)
            {
                throw new ArgumentException("Empty string specified.", "url");
            }

            this.url = url;
            if (title != null && title.Trim().Length == 0)
            {
                title = null;
            }
            this.title = title;
        }
    }
}
