/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 2.0
 */

using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Security.Permissions;
using System.Web;
using System.Web.UI;
using TopCoder.Util.ConfigurationManager;
using TopCoder.Util.ObjectFactory;

namespace TopCoder.Web.UI.WebControl.BreadCrumb
{
    /// <summary>
    /// The main tag class that application will use. This tag will render the current node to the root node (or
    /// vice-versa) using a path separator and specific styles. The application can setup the separator and the various
    /// styles either by specifying default values in the Configuration Manager or by specifying attributes on the tag.
    /// The object factory is used to discover the node, the path to the root and the formatting of the path using the
    /// using the key(s) specified in the tag.
    /// </summary>
    /// <remarks>
    /// <para>
    /// This class will be directly embedded in an html page and used to render the bread crumb trail.  The control
    /// supports IDE designer by exposing the various properties in attribute sheets. All attributes are also bindable
    /// to allow maximum flexibility. The user may override the path discovery process via the <see cref="DataSource"/>
    /// property to provide a custom path source (the path discovery process will be ignored then). The data source can
    /// be an <see cref="IList"/>/array of <see cref="BreadCrumbNode"/> instance, a <see cref="DataTable"/> with title/
    /// url columns or a <see cref="DataSet"/> with at least one <see cref="DataTable"/> with title/url columns.
    /// </para>
    /// <para>
    /// This class is thread safe in the context of the rendering process.
    /// </para>
    /// </remarks>
    [AspNetHostingPermission(SecurityAction.Demand, Level=AspNetHostingPermissionLevel.Minimal),
    AspNetHostingPermission(SecurityAction.InheritanceDemand, Level=AspNetHostingPermissionLevel.Minimal),
    DefaultProperty("PathSeparator"),
    ToolboxData("<{0}:BreadCrumbTrailControl runat=\"server\"> </{0}:BreadCrumbTrailControl>"),
    Designer(typeof(BreadCrumbTrailControlDesigner))]
    public class BreadCrumbTrailControl : System.Web.UI.WebControls.WebControl
    {
        /// <summary>
        /// The expected name of a database table column for document titles.
        /// </summary>
        private const string TITLE_COLUMN = "Title";

        /// <summary>
        /// The expected name of a database table column for document URLs.
        /// </summary>
        private const string URL_COLUMN = "Url";

        /// <summary>
        /// The private variable backing up <see cref="PathSeparator"/>.
        /// </summary>
        private string pathSeparator = ":";

        /// <summary>
        /// Represents the separator that will be drawn between nodes in a path. The default value is ":" and can be
        /// set in the constructor, is mutable and can be null to specify no path separator to use.
        /// </summary>
        /// <remarks>
        /// Empty strings are allowed in this property, in contrast to most other string properties in this control.
        /// </remarks>
        /// <value>
        /// null, or a string.
        /// </value>
        [Bindable(true),
        Category("Appearance"),
        DefaultValue(":"),
        Description("The path separator to use.")]
        public virtual string PathSeparator
        {
            get
            {
                return pathSeparator;
            }
            set
            {
                pathSeparator = value;
            }
        }

        /// <summary>
        /// The configuration manager key used for <see cref="pathSeparator"/>.
        /// </summary>
        private const string PATHSEPARATOR = "pathSeparator";

        /// <summary>
        /// The private variable backing up <see cref="PathSeparatorStyle"/>.
        /// </summary>
        private string pathSeparatorStyle = null;

        /// <summary>
        /// Represents the separator CSS style class that will be used to draw the separator.  It is assumed that the
        /// associated style class definitions (in the .css file) are included on the resulting webpage.  This
        /// variable is set initially to null and can be set in the constructor, is mutable and can be null to specify
        /// no path separator style to use.
        /// </summary>
        /// <remarks>
        /// In the setter, empty strings after trimming will be converted to null. <see cref="RenderContents"/> uses
        /// these values.
        /// </remarks>
        /// <value>
        /// null, or a non-empty string (after trimming).
        /// </value>
        [Bindable(true),
        Category("Appearance"),
        DefaultValue(""),
        Description("The CSS Style Class to use for the path separator.")]
        public virtual string PathSeparatorStyle
        {
            get
            {
                return pathSeparatorStyle;
            }
            set
            {
                if (value != null && value.Trim().Length == 0)
                {
                    value = null;
                }
                pathSeparatorStyle = value;
            }
        }

        /// <summary>
        /// The configuration manager key used for <see cref="pathSeparatorStyle"/>.
        /// </summary>
        private const string PATHSEPARATORSTYLE = "pathSeparatorStyle";

        /// <summary>
        /// The private variable backing up <see cref="NodeStyle"/>.
        /// </summary>
        private string nodeStyle = null;

        /// <summary>
        /// Represents the non-root, non-current node CSS style class that will be used to draw the node. It is assumed
        /// that the associated style class definitions (in the .css file) are included on the resulting webpage. This
        /// variable is set initially to null and can be set in the constructor, is mutable and can be null to specify
        /// no node style to use.
        /// </summary>
        /// <remarks>
        /// In the setter, empty strings after trimming will be converted to null. <see cref="RenderContents"/> uses
        /// these values.
        /// </remarks>
        /// <value>
        /// null, or a non-empty string (after trimming).
        /// </value>
        [Bindable(true),
        Category("Appearance"),
        DefaultValue(""),
        Description("The CSS Style Class to use for each node in the path.")]
        public virtual string NodeStyle
        {
            get
            {
                return nodeStyle;
            }
            set
            {
                if (value != null && value.Trim().Length == 0)
                {
                    value = null;
                }
                nodeStyle = value;
            }
        }

        /// <summary>
        /// The configuration manager key used for <see cref="nodeStyle"/>.
        /// </summary>
        private const string NODESTYLE = "nodeStyle";

        /// <summary>
        /// The private variable backing up <see cref="CurrentNodeStyle"/>.
        /// </summary>
        private string currentNodeStyle = null;

        /// <summary>
        /// Represents the current (only) node CSS style class that will be used to draw the current node. It is
        /// assumed that the associated style class definitions (in the .css file) are included on the resulting
        /// webpage. This field is set initially to null and can be set in the constructor, is mutable and can be
        /// null to specify no current node style to use.
        /// </summary>
        /// <remarks>
        /// In the setter, empty strings after trimming will be converted to null. <see cref="RenderContents"/> uses
        /// these values.
        /// </remarks>
        /// <value>
        /// null, or a non-empty string (after trimming).
        /// </value>
        [Bindable(true),
        Category("Appearance"),
        DefaultValue(""),
        Description("The CSS Style Class to use for the current node in the path.")]
        public virtual string CurrentNodeStyle
        {
            get
            {
                return currentNodeStyle;
            }
            set
            {
                if (value != null && value.Trim().Length == 0)
                {
                    value = null;
                }
                currentNodeStyle = value;
            }
        }

        /// <summary>
        /// The configuration manager key used for <see cref="currentNodeStyle"/>.
        /// </summary>
        private const string CURRENTNODESTYLE = "currentNodeStyle";

        /// <summary>
        /// The private variable backing up <see cref="RootNodeStyle"/>.
        /// </summary>
        private string rootNodeStyle = null;

        /// <summary>
        /// Represents the root (only) node CSS style class that will be used to draw the current node. It is assumed
        /// that the associated style class definitions (in the .css file) are included on the resulting webpage. This
        /// variable is set initially to null and can be set in the constructor, is mutable and can be null to specify
        /// no root node style to use.
        /// </summary>
        /// <remarks>
        /// In the setter, empty strings after trimming will be converted to null. <see cref="RenderContents"/> uses
        /// these values.
        /// </remarks>
        /// <value>
        /// null, or a non-empty string (after trimming).
        /// </value>
        [Bindable(true),
        Category("Appearance"),
        DefaultValue(""),
        Description("The CSS Style Class to use for the root node in the path.")]
        public virtual string RootNodeStyle
        {
            get
            {
                return rootNodeStyle;
            }
            set
            {
                if (value != null && value.Trim().Length == 0)
                {
                    value = null;
                }
                rootNodeStyle = value;
            }
        }

        /// <summary>
        /// The configuration manager key used for <see cref="rootNodeStyle"/>.
        /// </summary>
        private const string ROOTNODESTYLE = "rootNodeStyle";

        /// <summary>
        /// The private variable backing up <see cref="MouseOverStyle"/>.
        /// </summary>
        private string mouseOverStyle = null;

        /// <summary>
        /// Represents the CSS style class that will be used to change the nodes style when the mouse is over it.
        /// It is assumed that the associated style class definitions (in the .css file) are included on the resulting
        /// webpage. This field is set initially to null and can be set in the constructor, is mutable and can be
        /// null to specify no mouse over style to use.
        /// </summary>
        /// <remarks>
        /// In the setter, empty strings after trimming will be converted to null. <see cref="RenderContents"/> uses
        /// these values.
        /// </remarks>
        /// <value>
        /// null, or a non-empty string (after trimming).
        /// </value>
        [Bindable(true),
        Category("Appearance"),
        DefaultValue(""),
        Description("The CSS Style Class to use when the mouse hovers over a node in the path.")]
        public virtual string MouseOverStyle
        {
            get
            {
                return mouseOverStyle;
            }
            set
            {
                if (value != null && value.Trim().Length == 0)
                {
                    value = null;
                }
                mouseOverStyle = value;
            }
        }

        /// <summary>
        /// The configuration manager key used for <see cref="mouseOverStyle"/>.
        /// </summary>
        private const string MOUSEOVERSTYLE = "mouseOverStyle";

        /// <summary>
        /// The private variable backing up <see cref="ErrorDisposition"/>.
        /// </summary>
        private ErrorDisposition errorDisposition = ErrorDisposition.SilentlyIgnore;

        /// <summary>
        /// Represents whether any exceptions should be silently ignore, errors shown or exceptions to be thrown. This
        /// variable is set by the constructor, is mutable and can only be a valid enum value.
        /// </summary>
        /// <remarks>
        /// The default value is to silently ignore exceptions.
        /// </remarks>
        /// <exception cref="ArgumentException">If <paramref name="value"/> is not a valid enum value.</exception>
        /// <value>
        /// A valid <see cref="BreadCrumb.ErrorDisposition"/> value.
        /// </value>
        [Bindable(true),
        Category("Behavior"),
        DefaultValue(ErrorDisposition.SilentlyIgnore),
        Description("The disposition of any errors encountered during rendering.")]
        public virtual ErrorDisposition ErrorDisposition
        {
            get
            {
                return errorDisposition;
            }
            set
            {
                if (!Enum.IsDefined(typeof(ErrorDisposition), value))
                {
                    throw new ArgumentException("Invalid enum value.", "value");
                }
                errorDisposition = value;
            }
        }

        /// <summary>
        /// The configuration manager key used for <see cref="errorDisposition"/>.
        /// </summary>
        private const string ERRORDISPOSITION = "errorDisposition";

        /// <summary>
        /// The private variable backing up <see cref="PathDirection"/>.
        /// </summary>
        private PathDirection pathDirection = PathDirection.RootToCurrent;

        /// <summary>
        /// Represents the direction of the path that will be rendered. This field is set by the constructor, is
        /// mutable and can only be a valid enum value.
        /// </summary>
        /// <exception cref="ArgumentException">If <paramref name="value"/> is not a valid enum value.</exception>
        /// <value>
        /// A valid <see cref="BreadCrumb.PathDirection"/> value.
        /// </value>
        [Bindable(true),
        Category("Behavior"),
        DefaultValue(PathDirection.RootToCurrent),
        Description("The direction the path should be rendered.")]
        public virtual PathDirection PathDirection
        {
            get
            {
                return pathDirection;
            }
            set
            {
                if (!Enum.IsDefined(typeof(PathDirection), value))
                {
                    throw new ArgumentException("Invalid enum value.", "value");
                }
                pathDirection = value;
            }
        }

        /// <summary>
        /// The configuration manager key used for <see cref="pathDirection"/>.
        /// </summary>
        private const string PATHDIRECTION = "pathDirection";

        /// <summary>
        /// The private variable backing up <see cref="DataSource"/>.
        /// </summary>
        private object dataSource = null;

        /// <summary>
        /// Represents the data source to use (overriding the path discovery process) in determining the current path.
        /// The object passed can be an:
        /// <list>
        /// <item><see cref="IList"/> of <see cref="BreadCrumbNode"/> elements</item>
        /// <item>Array of <see cref="BreadCrumbNode"/> elements</item>
        /// <item>A <see cref="DataTable"/> that has columns of "Title" and "Url" defined in it</item>
        /// <item>A <see cref="DataSet"/> containing atleast one <c>DataTable</c> with columns defined above</item>
        /// </list>
        /// This field is initially to null, is mutable and can be null to specify no path override. The value is used
        /// in the <see cref="DataBind"/> property.
        /// </summary>
        /// <exception cref="ArgumentException">If <paramref name="value"/> is an <see cref="IList"/> (or array) with
        /// null elements, or being empty, or elements of any other type than <see cref="BreadCrumbNode"/>, or elements
        /// of that type with null URLs.</exception>
        /// <exception cref="ArgumentException">If <paramref name="value"/> is a <see cref="DataTable"/> missing
        /// either the Title or Url column.</exception>
        /// <exception cref="ArgumentException">If <paramref name="value"/> is a <see cref="DataSet"/> with no tables,
        /// or where the first table doesn't match the criteria for a <c>DataTable</c>.</exception>
        /// <exception cref="ArgumentException">If the value doesn't match any expected type.</exception>
        /// <value>
        /// An <c>IList</c>, <c>DataSet</c> or <c>DataTable</c> (see the requirements above).
        /// </value>
        [Bindable(true),
        Category("Data"),
        DefaultValue(null),
        Description("The override for the nodes in the path.")]
        public virtual object DataSource
        {
            get
            {
                return dataSource;
            }
            set
            {
                // This even includes BreadCrumbNode[] arrays.
                if (value is IList)
                {
                    if (((IList) value).Count == 0)
                    {
                        throw new ArgumentException("List empty.", "value");
                    }
                    try
                    {
                        foreach (BreadCrumbNode element in (IList) value)
                        {
                            if (element.Url == null)
                            {
                                throw new ArgumentException("Non-inited BreadCrumbNode present.", "value");
                            }
                        }
                    }
                    catch (NullReferenceException e)
                    {
                        throw new ArgumentException("Null element present.", "value", e);
                    }
                    catch (InvalidCastException e)
                    {
                        throw new ArgumentException("Element not of BreadCrumbNode type.", "value", e);
                    }

                    dataSource = value;
                    return;
                }

                // If we get a table, remember the original set
                object origValue = value;

                if (value is DataSet)
                {
                    DataSet set = (DataSet) value;
                    if (set.Tables.Count == 0)
                    {
                        throw new ArgumentException("No tables in data set.", "value");
                    }
                    value = set.Tables[0];
                }

                DataTable table = value as DataTable;
                if (table != null)
                {
                    if (!table.Columns.Contains(TITLE_COLUMN) || !table.Columns.Contains(URL_COLUMN))
                    {
                        throw new ArgumentException("Required columns missing for data set.", "value");
                    }

                    dataSource = origValue;
                    return;
                }

                if (value != null)
                {
                    throw new ArgumentException("Unknown data source type.", "value");
                }
                // Set back to null
                dataSource = null;
            }
        }

        /// <summary>
        /// The private variable backing up <see cref="Title"/>.
        /// </summary>
        private string title = null;

        /// <summary>
        /// Represents the title that is assigned to the page this control appears in. If no title is specified
        /// (null), the title found in the static web site will be used instead (if none is found, "Unknown" will be
        /// rendered instead). This field is set initially to null, is mutable and can be null to specify no specific
        /// title to use (with the mentioned rules applied).
        /// </summary>
        /// <remarks>
        /// In the setter, empty strings after trimming will be converted to null. <see cref="RenderContents"/> uses
        /// these values.
        /// </remarks>
        /// <value>
        /// null, or a non-empty string (after trimming).
        /// </value>
        [Bindable(true),
        Category("Appearance"),
        DefaultValue(""),
        Description("The title override to use for the current page.")]
        public virtual string Title
        {
            get
            {
                return title;
            }
            set
            {
                if (value != null && value.Trim().Length == 0)
                {
                    value = null;
                }
                title = value;
            }
        }

        /// <summary>
        /// The configuration manager key used for <see cref="title"/>.
        /// </summary>
        private const string TITLE = "title";

        /// <summary>
        /// The private variable backing up <see cref="NodeDiscoveryKey"/>.
        /// </summary>
        private string nodeDiscoveryKey = null;

        /// <summary>
        /// Represents the Object Factory key that will be used when the <see cref="INodeDiscovery"/> implementation is
        /// created. If no key is specified (i.e. null), the <c>INodeDiscovery</c> type's full name will be used
        /// instead. This field is set initially to null and can be set in the constructor, is mutable and can be null.
        /// </summary>
        /// <remarks>
        /// In the setter, empty strings after trimming will be converted to null.
        /// </remarks>
        /// <value>
        /// null, or a non-empty string (after trimming).
        /// </value>
        [Bindable(true),
        Category("Behavior"),
        DefaultValue(""),
        Description("The Object Factory key to use in creating the INodeDiscovery implementation.")]
        public virtual string NodeDiscoveryKey
        {
            get
            {
                return nodeDiscoveryKey;
            }
            set
            {
                if (value != null && value.Trim().Length == 0)
                {
                    value = null;
                }
                nodeDiscoveryKey = value;
            }
        }

        /// <summary>
        /// The configuration manager key used for <see cref="nodeDiscoveryKey"/>.
        /// </summary>
        private const string NODEDISCOVERYKEY = "nodeDiscoveryKey";

        /// <summary>
        /// The private variable backing up <see cref="PathDiscoveryKey"/>.
        /// </summary>
        private string pathDiscoveryKey = null;

        /// <summary>
        /// Represents the Object Factory key that will be used when the <see cref="IPathDiscovery"/> implementation is
        /// created. If no key is specified (i.e. null), the <c>IPathDiscovery</c> type's full name will be used
        /// instead. This field is set initially to null and can be set in the constructor, is mutable and can be null.
        /// </summary>
        /// <remarks>
        /// In the setter, empty strings after trimming will be converted to null.
        /// </remarks>
        /// <value>
        /// null, or a non-empty string (after trimming).
        /// </value>
        [Bindable(true),
        Category("Behavior"),
        DefaultValue(""),
        Description("The Object Factory key to use in creating the IPathDiscovery implementation.")]
        public virtual string PathDiscoveryKey
        {
            get
            {
                return pathDiscoveryKey;
            }
            set
            {
                if (value != null && value.Trim().Length == 0)
                {
                    value = null;
                }
                pathDiscoveryKey = value;
            }
        }

        /// <summary>
        /// The configuration manager key used for <see cref="pathDiscoveryKey"/>.
        /// </summary>
        private const string PATHDISCOVERYKEY = "pathDiscoveryKey";

        /// <summary>
        /// The private variable backing up <see cref="PathWriterKey"/>.
        /// </summary>
        private string pathWriterKey = null;

        /// <summary>
        /// Represents the Object Factory key that will be used when the <see cref="IPathWriter"/> implementation is
        /// created. If no key is specified (i.e. null), the <c>IPathWriter</c> type's full name will be used
        /// instead. This field is set initially to null and can be set in the constructor, is mutable and can be null.
        /// </summary>
        /// <remarks>
        /// In the setter, empty strings after trimming will be converted to null.
        /// </remarks>
        /// <value>
        /// null, or a non-empty string (after trimming).
        /// </value>
        [Bindable(true),
        Category("Behavior"),
        DefaultValue(""),
        Description("The Object Factory key to use in creating the IPathWriter implementation.")]
        public virtual string PathWriterKey
        {
            get
            {
                return pathWriterKey;
            }
            set
            {
                if (value != null && value.Trim().Length == 0)
                {
                    value = null;
                }
                pathWriterKey = value;
            }
        }

        /// <summary>
        /// The configuration manager key used for <see cref="pathWriterKey"/>.
        /// </summary>
        private const string PATHWRITERKEY = "pathWriterKey";

        /// <summary>
        /// Represents the path override that should be used (in place of the path discovery process). This field is
        /// initially set to null, is mutable and can be null (but not an empty size) if no path override is in place.
        /// This field is only set to non-null in the <see cref="DataBind"/> method if a
        /// <see cref="dataSource"/> has been specified.  The variable is then referenced in the
        /// <see cref="RenderContents"/> method to determine whether the path discovery process should be used (if this
        /// variable is null) or not (if this field is non-null).
        /// </summary>
        private BreadCrumbNode[] pathOverride = null;

        /// <summary>
        /// The namespace used for loading the configuration.
        /// </summary>
        private static readonly string NAMESPACE = typeof(BreadCrumbTrailControl).FullName;

        /// <summary>
        /// Reads a value from a configuration manager namespace. If the value is present, AND not null AND not empty,
        /// after trimming, it is stored into <paramref name="value"/>, otherwise the default/previous value is
        /// preserved.
        /// </summary>
        /// <param name="ns">The configuration manager namespace.</param>
        /// <param name="propertyName">The property name.</param>
        /// <param name="acceptEmpty">Whether empty strings should be accepted and stored.</param>
        /// <param name="value">The variable to store the value into.</param>
        private static void ReadStringValue(Namespace ns, string propertyName, bool acceptEmpty, ref string value)
        {
            // Ignore any exceptions
            try
            {
                Property prop = ns.GetProperty(propertyName);
                if (prop != null)
                {
                    if (prop.Value != null && (acceptEmpty || prop.Value.Trim().Length != 0))
                    {
                        value = prop.Value;
                    }
                }
            }
            catch (Exception)
            {
            }
        }

        /// <summary>
        /// Reads a value from a configuration manager namespace. If the value is present, AND not null AND not empty,
        /// after trimming, it is parsed into the relevant type and stored in value.
        /// </summary>
        /// <param name="ns">The configuration manager namespace.</param>
        /// <param name="enumType">The enumeration type to parse to.</param>
        /// <param name="propertyName">The property name.</param>
        /// <param name="value">The previous value, to return if no new value is found.</param>
        /// <returns>
        /// The updated value.
        /// </returns>
        private static object ReadEnumValue(Namespace ns, Type enumType, string propertyName, object value)
        {
            // Ignore any exceptions
            try
            {
                Property prop = ns.GetProperty(propertyName);
                if (prop != null)
                {
                    if (prop.Value != null && prop.Value.Trim().Length != 0)
                    {
                        value = Enum.Parse(enumType, prop.Value);
                    }
                }
            }
            catch (Exception)
            {
            }

            return value;
        }


        /// <summary>
        /// Load all configuration from a namespace based on the full name of this type. See the component spec for
        /// more information regarding the configuration. Basically, all properties that are present are loaded, but
        /// any that are not get their defaults.
        /// </summary>
        public BreadCrumbTrailControl()
        {
            try
            {
                Namespace ns = ConfigManager.GetInstance().GetNamespace(NAMESPACE);
                ReadStringValue(ns, PATHSEPARATOR, true, ref pathSeparator);
                ReadStringValue(ns, PATHSEPARATORSTYLE, false, ref pathSeparatorStyle);
                ReadStringValue(ns, NODESTYLE, false, ref nodeStyle);
                ReadStringValue(ns, CURRENTNODESTYLE, false, ref currentNodeStyle);
                ReadStringValue(ns, ROOTNODESTYLE, false, ref rootNodeStyle);
                ReadStringValue(ns, MOUSEOVERSTYLE, false, ref mouseOverStyle);
                errorDisposition = (ErrorDisposition) ReadEnumValue(ns, typeof(ErrorDisposition), ERRORDISPOSITION,
                    errorDisposition);
                pathDirection = (PathDirection) ReadEnumValue(ns, typeof(PathDirection), PATHDIRECTION, pathDirection);
                ReadStringValue(ns, TITLE, false, ref title);
                ReadStringValue(ns, NODEDISCOVERYKEY, false, ref nodeDiscoveryKey);
                ReadStringValue(ns, PATHDISCOVERYKEY, false, ref pathDiscoveryKey);
                ReadStringValue(ns, PATHWRITERKEY, false, ref pathWriterKey);
            }
            catch (Exception)
            {
            }
        }

        /// <summary>
        /// The data binding performed by this component consists of using <see cref="DataSource"/>, and use that as
        /// the breadcrumb trail path, if anything is provided. If not, the path discovery mechanism is used.
        /// </summary>
        /// <exception cref="ArgumentException">If <see cref="DataSource"/> is an <see cref="IList"/> (or array) with
        /// null elements, or being empty, or elements of any other type than <see cref="BreadCrumbNode"/>, or elements
        /// of that type with null URLs.</exception>
        /// <exception cref="ArgumentException">If <see cref="DataSource"/> is a <see cref="DataTable"/> missing
        /// either the Title or Url column.</exception>
        /// <exception cref="ArgumentException">If <see cref="DataSource"/> is a <see cref="DataSet"/> with no tables,
        /// or where the first table doesn't match the criteria for a <c>DataTable</c>.</exception>
        /// <exception cref="ArgumentException">If the value doesn't match any expected type.</exception>
        /// <exception cref="ArgumentException">Any other error while parsing the data source (for example null/empty
        /// URLs in a data source, or unexpected/invalid types).</exception>
        public override void DataBind()
        {
            base.DataBind();
            pathOverride = null;

            // By calling the setter again, we make sure that we throw for any of the error conditions.
            this.DataSource = dataSource;

            try
            {
                // This even includes BreadCrumbNode[] arrays.
                IList list = DataSource as IList;
                if (list != null)
                {
                    pathOverride = new BreadCrumbNode[list.Count];
                    list.CopyTo(pathOverride, 0);
                }
                if (DataSource is DataSet)
                {
                    DataSet set = (DataSet) DataSource;
                    if (set.Tables.Count == 0)
                    {
                        throw new ArgumentException("No tables in data set.", "DataSource");
                    }
                    DataSource = set.Tables[0];
                }

                DataTable table = DataSource as DataTable;
                if (table != null)
                {
                    DataRowCollection rows = table.Rows;
                    DataColumn titleColumn = table.Columns[TITLE_COLUMN];
                    DataColumn urlColumn = table.Columns[URL_COLUMN];

                    pathOverride = new BreadCrumbNode[rows.Count];
                    for (int i = 0; i < rows.Count; i++)
                    {
                        pathOverride[i] = new BreadCrumbNode((string) rows[i][titleColumn],
                            (string) rows[i][urlColumn]);
                    }
                }

                if (pathOverride != null && pathOverride.Length == 0)
                {
                    pathOverride = null;
                }
            }
            catch (Exception e)
            {
                pathOverride = null;
                throw new ArgumentException("Data source error.", e);
            }
        }

        /// <summary>
        /// A simple helper that wraps an exception if it's not already a <see cref="BreadCrumbException"/>.
        /// </summary>
        /// <param name="e">The exception to (possibly) wrap.</param>
        /// <returns>The resulting exception.</returns>
        private static BreadCrumbException WrapIfNeeded(Exception e)
        {
            if (e is BreadCrumbException)
            {
                return (BreadCrumbException) e;
            }

            return new BreadCrumbException("Error encountered.", e);
        }

        /// <summary>
        /// Creates an object from the object factory, with key <paramref name="key"/>, if that's not null. If it is,
        /// the default value is used instead.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <param name="_default">The default value.</param>
        /// <returns>The created object</returns>
        private object CreateObject(string key, string _default)
        {
            if (key == null)
            {
                key = _default;
            }

            try
            {
                return ObjectFactory.GetDefaultObjectFactory().CreateDefinedObject(key);
            }
            catch (Exception e)
            {
                throw new BreadCrumbException(string.Format("Object factory error: {0}", key), e);
            }
        }

        /// <summary>
        /// Writes the path, by using the path override in the datasource if provided, and otherwise the
        /// system of node discovery, path discovery. In all cases, the path writer is used. If the relevant properties
        /// are empty, the default values for the object factory keys to those classes are the full names of the
        /// relevant interfaces, <see cref="IPathWriter"/>, <see cref="IPathDiscovery"/>, <see cref="INodeDiscovery"/>.
        /// </summary>
        /// <param name="writer">The writer to use.</param>
        /// <exception cref="ArgumentNullException">If <paramref name="writer"/> is null.</exception>
        /// <exception cref="BreadCrumbException">Any error, if <see cref="ErrorDisposition"/> is
        /// <c>ErrorDisposition.ThrowException</c>.</exception>
        protected override void RenderContents(HtmlTextWriter writer)
        {
            if (writer == null)
            {
                throw new ArgumentNullException("writer");
            }

            BreadCrumbException[] exceptions = null;

            try
            {
                BreadCrumbNode[] nodes = null;
                if (pathOverride == null)
                {
                    // Do node and path discovery
                    INodeDiscovery nodeFinder = (INodeDiscovery)
                            CreateObject(nodeDiscoveryKey, typeof(INodeDiscovery).FullName);

                    BreadCrumbNode thisNode = nodeFinder.GetCurrentNode(title, Page);

                    IPathDiscovery pathFinder =
                        (IPathDiscovery) CreateObject(pathDiscoveryKey, typeof(IPathDiscovery).FullName);
                    nodes = pathFinder.GetPath(thisNode, Page);
                }
                else
                {
                    nodes = pathOverride;
                }

                // If there is no trail, stay quiet
                if (nodes == null || nodes.Length == 0)
                {
                    return;
                }

                IPathWriter pathWriter =
                        (IPathWriter) CreateObject(pathWriterKey, typeof(IPathWriter).FullName);


                exceptions = pathWriter.WritePath(writer, nodes, this);
            }
            catch (Exception e)
            {
                BreadCrumbException bce = WrapIfNeeded(e);

                exceptions = new BreadCrumbException[] { bce };
            }

            // Handle any exceptions
            if (exceptions != null && exceptions.Length > 0)
            {
                switch (ErrorDisposition)
                {
                    case ErrorDisposition.SilentlyIgnore:
                        // Nothing
                        break;
                    case ErrorDisposition.RenderException:
                        // Render each exception, but throw nothing
                        foreach (BreadCrumbException bce in exceptions)
                        {
                            writer.RenderBeginTag(HtmlTextWriterTag.P);
                            writer.WriteLine(HttpUtility.HtmlEncode(bce.ToString()));
                            writer.RenderEndTag();
                        }
                        break;
                    case ErrorDisposition.ThrowException:
                        // Throw the first (and if done correctly, only) one.
                        throw exceptions[0];
                }
            }
        }
    }
}
