/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 2.0
 */

using System;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery
{
    /// <summary>
    /// This abstract implementation of the <see cref="INodeMatcher"/> interface provides title and url type services
    /// to subclasses. This implementation will provide a title and URL holder variable and getters.
    /// </summary>
    /// <remarks>
    /// It's recommended that any <c>INodeMatcher</c> implementation is made by subclassing this class. That ensures
    /// continued flexibility in later versions. For now, this abstract base class only implements title/url handling,
    /// but this may be extended in a later release.
    /// </remarks>
    [Serializable]
    public abstract class AbstractNodeMatcher : INodeMatcher
    {
        /// <summary>
        /// Represents the default title that a node can take if it matches this node matcher. This variable is set in
        /// the constructor, is immutable and can never be null nor an empty string (after trimming). The title is
        /// referenced in the associated property getter.
        /// </summary>
        private string title;

        /// <summary>
        /// Represents the default URL that can be used if not contextual information can be used. This field is set in
        /// the constructor, is immutable and can never be null nor an empty string (after trimming). The URL is
        /// referenced in the associated property getter.
        /// </summary>
        private string url;

        /// <summary>
        /// Represents the default title that a node can take if it matches this node matcher. This variable is set
        /// in the constructor, is immutable and can never be null nor an empty string (after trimming).
        /// </summary>
        /// <value>
        /// A title string, not null, not empty.
        /// </value>
        public string Title
        {
            get
            {
                return title;
            }
        }

        /// <summary>
        /// Represents the default URL that can be used if not contextual information can be used. This variable is set
        /// in the constructor, is immutable and can never be null nor an empty string (after trimming). The URL is
        /// referenced in the associated property getter.
        /// </summary>
        /// <value>
        /// A URL, not null, not empty.
        /// </value>
        public string Url
        {
            get
            {
                return url;
            }
        }

        /// <summary>
        /// Constructs the abstract base from the given title and URL.
        /// </summary>
        /// <param name="title">A non-null, non-empty string (after trimming) to use for title.</param>
        /// <param name="url">A non-null, non-empty string (after trimming) to use for URL.</param>
        /// <exception cref="ArgumentNullException">If <paramref name="title"/> or <paramref name="url"/> is null.
        /// </exception>
        /// <exception cref="ArgumentException">If <paramref name="title"/> or <paramref name="url"/> is empty.
        /// </exception>
        protected AbstractNodeMatcher(string title, string url)
        {
            if (title == null)
            {
                throw new ArgumentNullException("title");
            }
            if (title.Trim().Length == 0)
            {
                throw new ArgumentException("Empty string specified.", "title");
            }

            if (url == null)
            {
                throw new ArgumentNullException("url");
            }
            if (url.Trim().Length == 0)
            {
                throw new ArgumentException("Empty string specified.", "url");
            }

            this.url = url;
            this.title = title;
        }

        /// <summary>
        /// Determines whether the specified node matches this node matcher.
        /// </summary>
        /// <param name="node">A non-null node to test.</param>
        /// <exception cref="ArgumentNullException">If <paramref name="node"/> has a null URL.</exception>
        /// <returns><c>true</c> or <c>false</c></returns>
        public abstract bool Matches(BreadCrumbNode node);
    }
}
