/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 2.0
 */

using System;
using System.Collections;
using System.Web.UI;
using TopCoder.Util.ObjectFactory;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery
{
    /// <summary>
    /// This implementation of <c>IPathDiscovery</c> will attempt to create a static site map and then apply dynamic
    /// information to that site map. The dynamic information can either be in matching nodes with different query
    /// strings or in new nodes that will be assumed to map to the last known node.
    /// </summary>
    /// <remarks>
    /// <para>
    /// This implementation will use a <see cref="ISiteDiscovery"/> to discover the site map. That site map will then
    /// be used to help determine where the current node belongs and what it's path back to the root node is. Please
    /// refer to the component specification for the exact algorithm this class implements.
    /// </para>
    /// <para>
    /// This class is thread safe in context of the rendering process. A static session field is used for the site map
    /// after creation, for performance reasons. The current path is also stored for dynamic capabilities. A thread
    /// safety analysis of these facts is present in the component specification.
    /// </para>
    /// </remarks>
    public class DynamicPathDiscovery : IPathDiscovery
    {
        /// <summary>
        /// A "constant" referencing the name of the session variable where the static site map is stored. Direct
        /// access to it is not recommended, but could for example be used as a way to force the path discovery class
        /// to refresh the map data.
        /// </summary>
        public static readonly string CURRENT_SITEMAP = typeof(DynamicPathDiscovery).FullName + ".siteMap";

        /// <summary>
        /// A "constant" referencing the name of the session variable where the static reversed site map is stored.
        /// Direct access to it is not recommended, but could for example be used as a way to force the path discovery
        /// class to refresh the map data.
        /// </summary>
        /// <remarks>
        /// The value is static readonly instead of a true const, to make sure that the value will stay current, even
        /// if the namespace name would change.
        /// </remarks>
        public static readonly string REVERSED_SITEMAP = typeof(DynamicPathDiscovery).FullName + ".reversedSiteMap";

        /// <summary>
        /// A "constant" referencing the name of the session variable where the (previous) path is stored, in a
        /// <see cref="NodePair"/> array representation.
        /// </summary>
        public static readonly string CURRENT_PATH =  typeof(DynamicPathDiscovery).FullName + ".currentPath";

        /// <summary>
        /// An empty constructor used as a place holder for future logic.
        /// </summary>
        public DynamicPathDiscovery()
        {
        }

        /// <summary>
        /// This method attempts to determine the page from the current node to the root and return that path based on
        /// the current site map and previous path.
        /// </summary>
        /// <remarks>
        /// The current sitemap is retrieved with the session attribute <see cref="CURRENT_SITEMAP"/>. The path is
        /// retrieved by <see cref="CURRENT_PATH"/>.
        /// </remarks>
        /// <param name="currentNode">A non-null node representing the current page.</param>
        /// <param name="page">The non-null page context.</param>
        /// <exception cref="ArgumentNullException">If <paramref name="currentNode"/> has a null URL or
        /// <paramref name="page"/> is null.</exception>
        /// <exception cref="BreadCrumbException">For any exception while discovering the path.</exception>
        /// <returns>A <see cref="BreadCrumbNode"/> array describing a path from root to current node.</returns>
        public BreadCrumbNode[] GetPath(BreadCrumbNode currentNode, Page page)
        {
            if (currentNode.Url == null)
            {
                throw new ArgumentNullException("currentNode", "BreadCrumbNode not properly initialized.");
            }
            if (page == null)
            {
                throw new ArgumentNullException("page");
            }

            try
            {
                IDictionary sitemap = page.Session[CURRENT_SITEMAP] as IDictionary;
                IDictionary reversedMap = page.Session[REVERSED_SITEMAP] as IDictionary;
                NodePair[] path = page.Session[CURRENT_PATH] as NodePair[];

                // Ok, we don't have all the needed cached data. Reread and recompute.
                if (sitemap == null || reversedMap == null)
                {
                    // Use the object factory, for this type.
                    ISiteDiscovery siteDiscovery = (ISiteDiscovery) ObjectFactory.GetDefaultObjectFactory().
                        CreateDefinedObject(typeof(ISiteDiscovery).FullName);
                    sitemap = siteDiscovery.GetSiteMap();

                    reversedMap = PathDiscoveryHelper.GetReversedMap(sitemap);
                    page.Session[CURRENT_SITEMAP] = sitemap;
                    page.Session[REVERSED_SITEMAP] = reversedMap;
                    path = null;

                    // Reset the path if we reread all other data.
                }

                // Retrieve a path
                NodePair[] newPath = PathDiscoveryHelper.GetPath(currentNode, path, sitemap, reversedMap);

                // If we got no new path from the helper, do our completely dynamic part, that is, just append
                if (newPath == null)
                {
                    if (path != null)
                    {
                        newPath = new NodePair[path.Length + 1];
                        path.CopyTo(newPath, 0);
                    }
                    else
                    {
                        // One single entry
                        newPath = new NodePair[1];
                    }

                    newPath[newPath.Length - 1] = new NodePair(currentNode, null);
                }

                // Set the new path in the session
                page.Session[CURRENT_PATH] = newPath;

                // "Simplify" it into a BreadCrumbNode array.
                return PathDiscoveryHelper.SimplifyPath(newPath);
            }
            catch (BreadCrumbException)
            {
                throw;
            }
            catch (Exception e)
            {
                throw new BreadCrumbException("Error retrieving path.", e);
            }
        }
    }
}
