/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 2.0
 */

using System;
using System.Web.UI;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.PathWriter
{
    /// <summary>
    /// This path writer will write the node and title using the ordered list pattern where each rendered node is a
    /// separate line item.
    /// </summary>
    /// <remarks>
    /// This path writer will be created by the Object Factory and utilized by the <see cref="BreadCrumbTrailControl"/>
    /// to write the current node/title using an html anchor. This class is thread safe in the context of the rendering
    /// process.
    /// </remarks>
    public class OrderedListPathWriter : AnchorPathWriter
    {
        /// <summary>
        /// Constructs the <see cref="OrderedListPathWriter"/> using no constructor parameters.
        /// </summary>
        public OrderedListPathWriter()
            : base()
        {
        }

        /// <summary>
        /// Writes the beginning tag for the ordered list.
        /// </summary>
        /// <param name="writer">A non-null HTML writer.</param>
        /// <exception cref="ArgumentNullException">If <paramref name="writer"/> is null.</exception>
        /// <exception cref="BreadCrumbException">Any error.</exception>
        protected override void BeginPath(HtmlTextWriter writer)
        {
            base.BeginPath(writer);

            try
            {
                writer.RenderBeginTag(HtmlTextWriterTag.Ol);
            }
            catch (Exception e)
            {
                throw new BreadCrumbException("Error writing HTML.", e);
            }
        }

        /// <summary>
        /// This method will write the specified node and style as a simple HTML reference (a href), surrounded in a
        /// list item tag (li).
        /// </summary>
        /// <param name="writer">A non-null writer to use.</param>
        /// <param name="node">A non-null node to write.</param>
        /// <param name="nodeStyle">A style class to use for the node or null if no style should be used.</param>
        /// <param name="mouseOverStyle">The mouse over style class to use or null for none.</param>
        /// <exception cref="ArgumentNullException">If <paramref name="writer"/> or the URL of <paramref name="node"/>
        /// is null.
        /// </exception>
        /// <exception cref="BreadCrumbException">Any exception while writing.</exception>
        protected override void WriteNode(HtmlTextWriter writer, BreadCrumbNode node, string nodeStyle,
            string mouseOverStyle)
        {
            if (node.Url == null)
            {
                throw new ArgumentNullException("node", "BreadCrumbNode not properly initialized.");
            }
            if (writer == null)
            {
                throw new ArgumentNullException("writer");
            }

            try
            {
                writer.RenderBeginTag(HtmlTextWriterTag.Li);
                base.WriteNode(writer, node, nodeStyle, mouseOverStyle);
                writer.RenderEndTag();
            }
            catch (BreadCrumbException)
            {
                throw;
            }
            catch (Exception e)
            {
                throw new BreadCrumbException("Error writing HTML.", e);
            }
        }

        /// <summary>
        /// Writes no separator, even if specified, as we use the list layout instead.
        /// </summary>
        /// <param name="writer">A non-null writer to use.</param>
        /// <param name="parent">A non-null parent control to use.</param>
        /// <exception cref="ArgumentNullException">If <paramref name="writer"/> or <paramref name="parent"/> is null.
        /// </exception>
        protected override void WriteSeparator(HtmlTextWriter writer, BreadCrumbTrailControl parent)
        {
            if (writer == null)
            {
                throw new ArgumentNullException("writer");
            }
            if (parent == null)
            {
                throw new ArgumentNullException("parent");
            }
        }

        /// <summary>
        /// Writes the ending tag for the ordered list.
        /// </summary>
        /// <param name="writer">A non-null HTML writer.</param>
        /// <exception cref="ArgumentNullException">If <paramref name="writer"/> is null.</exception>
        /// <exception cref="BreadCrumbException">Any error.</exception>
        protected override void EndPath(HtmlTextWriter writer)
        {
            base.EndPath(writer);

            try
            {
                writer.RenderEndTag();
            }
            catch (Exception e)
            {
                throw new BreadCrumbException("Error writing HTML.", e);
            }
        }
    }
}
