/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER
 * Version: 2.0
 */

using System;
using NUnit.Framework;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery.NodeMatcher
{
    /// <summary>
    /// This class tests the regex node matcher. The constructor and Title/Url property functionality from
    /// <see cref="AbstractNodeMatcher"/> are also tested. The constructor arguments from the regex node matcher are
    /// essentially passed on to the abstract class.
    /// </summary>
    [TestFixture]
    public class TestRegexNodeMatcher
    {
        /// <summary>
        /// A test object array.
        /// </summary>
        private RegexNodeMatcher[] testObjects = new RegexNodeMatcher[2];

        /// <summary>
        /// This subclass is just used for verifying how equality tests are implemented.
        /// </summary>
        private class SubClassMatcher : RegexNodeMatcher
        {
            /// <summary>
            /// Constructor calling the base constructor, that's the only thing needed.
            /// </summary>
            /// <param name="title">The title string.</param>
            /// <param name="url">The URL string.</param>
            /// <param name="nodePattern">The regular expression pattern.</param>
            public SubClassMatcher(string title, string url, string nodePattern)
                : base(title, url, nodePattern)
            {
            }
        }

        /// <summary>
        /// Sets up the test objects, one through deserialization.
        /// </summary>
        [SetUp]
        public void Setup()
        {
            testObjects[0] = new RegexNodeMatcher("title", "url", "/");
            testObjects[1] = (RegexNodeMatcher) Helper.DeSerialize(testObjects[0]);
        }

        /// <summary>
        /// Tests the URL readonly property, verifying the value.
        /// </summary>
        [Test]
        public void TestUrl()
        {
            foreach (RegexNodeMatcher matcher in testObjects)
            {
                Assert.AreEqual("url", matcher.Url, "Url should match.");
            }
        }

        /// <summary>
        /// Tests the title readonly property, verifying the value.
        /// </summary>
        [Test]
        public void TestTitle()
        {
            foreach (RegexNodeMatcher matcher in testObjects)
            {
                Assert.AreEqual("title", matcher.Title, "Title should match.");
            }
        }

        /// <summary>
        /// Tests the pattern readonly property, verifying the value.
        /// </summary>
        [Test]
        public void TestNodePattern()
        {
            foreach (RegexNodeMatcher matcher in testObjects)
            {
                Assert.AreEqual("/", matcher.NodePattern, "Regex pattern should match.");
            }
        }

        /// <summary>
        /// Tests the equality method, including that we won't report identity against a subclass.
        /// </summary>
        [Test]
        public void TestEquals()
        {
            RegexNodeMatcher identical = new RegexNodeMatcher("title", "url", "/");
            RegexNodeMatcher different = new RegexNodeMatcher("title2", "url", "/");
            SubClassMatcher sameDataDifferentClass = new SubClassMatcher("title", "url", "/");

            foreach (RegexNodeMatcher matcher in testObjects)
            {
                Assert.IsFalse(matcher.Equals(different), "Comparison against a different object should fail.");
                Assert.IsTrue(matcher.Equals(identical), "Comparison against an identical object should succeed.");
                Assert.IsFalse(matcher.Equals(sameDataDifferentClass),
                    "Comparison against a different class should always fail.");

                Assert.IsFalse(matcher.Equals(null), "Comparison against null should give false.");
                Assert.IsFalse(matcher.Equals("a"), "Comparison against a string should give false.");
            }

            Assert.IsTrue(sameDataDifferentClass.Equals(sameDataDifferentClass),
                "Subclass compared against subclass should succeed.");
        }

        /// <summary>
        /// Tests the hashing method. We will report identity against a subclass, as hashes are not really completely
        /// unique.
        /// </summary>
        [Test]
        public void TestGetHashCode()
        {
            RegexNodeMatcher identical = new RegexNodeMatcher("title", "url", "/");
            RegexNodeMatcher different = new RegexNodeMatcher("title2", "url", "/");

            foreach (RegexNodeMatcher matcher in testObjects)
            {
                Assert.AreEqual(identical.GetHashCode(), matcher.GetHashCode(),
                    "Hash codes should match for identical objects.");
                Assert.IsFalse(different.GetHashCode() == matcher.GetHashCode(),
                    "Hash codes should not match for different objects.");
            }
        }

        /// <summary>
        /// Tests matching, including that the full string must match.
        /// </summary>
        [Test]
        public void TestMatches()
        {
            foreach (RegexNodeMatcher matcher in testObjects)
            {
                Assert.IsTrue(matcher.Matches(new BreadCrumbNode(null, "/")), "Correct string should match.");
                Assert.IsFalse(matcher.Matches(new BreadCrumbNode(null, "/\n/")),
                    "Several-line string should not match");
                Assert.IsFalse(matcher.Matches(new BreadCrumbNode(null, "a/b")), "The complete string should match.");
            }
        }

        /// <summary>
        /// Tests null constructor parameters. Exception expected.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructorNull1()
        {
            new RegexNodeMatcher(null, "url", "pattern");
        }

        /// <summary>
        /// Tests null constructor parameters. Exception expected.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructorNull2()
        {
            new RegexNodeMatcher("title", null, "pattern");
        }

        /// <summary>
        /// Tests null constructor parameters. Exception expected.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructorNull3()
        {
            new RegexNodeMatcher("title", "url", null);
        }

        /// <summary>
        /// Tests empty constructor parameters. Exception expected.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructorEmpty1()
        {
            new RegexNodeMatcher(" ", "url", "pattern");
        }

        /// <summary>
        /// Tests empty constructor parameters. Exception expected.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructorEmpty2()
        {
            new RegexNodeMatcher("title", " ", "pattern");
        }

        /// <summary>
        /// Tests empty constructor parameters. Exception expected.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructorEmpty3()
        {
            new RegexNodeMatcher("title", "url", " ");
        }
    }
}
