/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER
 * Version: 2.0
 */

using NUnit.Framework;
using System;
using System.Collections;
using System.Web.UI;
using TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery.NodeMatcher;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery
{
    /// <summary>
    /// This class tests <see cref="XmlPathDiscovery"/>.
    /// </summary>
    [TestFixture]
    public class TestXmlPathDiscovery
    {
        /// <summary>
        /// An object used during testing, with a rather simple tree structure.
        /// </summary>
        private XmlPathDiscovery testObject;

        /// <summary>
        /// Sets up the testing object, with some sample XML. It's placed here, in source, to be easy to relate to the
        /// tests. File-based loading is also tested, in order to verify that it works.
        /// </summary>
        [SetUp]
        public void Setup()
        {
            testObject = new XmlPathDiscovery(
                @"<BreadCrumb>
                    <bc_node title=""Main"" url=""/"" pattern=""/"">
                        <bc_node title=""Child1"" url=""/child1"" pattern=""/child1"">
                            <bc_node title=""ChildFolder"" url=""/childFolder/"" pattern=""/childFolder/.*"">
                                <bc_node title=""Child3"" url=""/child3"" pattern=""/child3"">
                                </bc_node>
                                <bc_node title=""Child4"" url=""/child4"" pattern=""/child4"">
                                </bc_node>
                            </bc_node>
                        </bc_node>
                        <bc_node title=""Child2"" url=""/child2"" pattern=""/child2"">
                        </bc_node>
                    </bc_node>
                    <bc_node title=""Alternative root"" url=""/other"" pattern=""/other"">
                        <bc_node title=""ChildFolder"" url=""/childFolder/"" pattern=""/childFolder/.*"" />
                    </bc_node>
                  </BreadCrumb>", false);
        }

        /// <summary>
        /// Tests the single-parameter constructor with a null string. An exception is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructorNull1()
        {
            new XmlPathDiscovery(null);
        }

        /// <summary>
        /// Tests the two-parameter constructor with a null string. An exception is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructorNull2()
        {
            new XmlPathDiscovery(null, false);
        }

        /// <summary>
        /// Tests the single-parameter constructor with an empty string. An exception is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructorEmpty1()
        {
            new XmlPathDiscovery(" ");
        }

        /// <summary>
        /// Tests the two-parameter constructor with an empty. An exception is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructorEmpty2()
        {
            new XmlPathDiscovery(" ", false);
        }

        /// <summary>
        /// Tests constructing with some invalid XML (incorrect root element).
        /// </summary>
        [ExpectedException(typeof(BreadCrumbException))]
        [Test]
        public void TestInvalidXml1()
        {
            new XmlPathDiscovery("<breadcrumb></breadcrumb>", false).GetSiteMap();
        }

        /// <summary>
        /// Tests constructing with some invalid XML (invalid additional child).
        /// </summary>
        [ExpectedException(typeof(BreadCrumbException))]
        [Test]
        public void TestInvalidXml2()
        {
            new XmlPathDiscovery("<BreadCrumb><bc_node><b/></bc_node></BreadCrumb>", false).GetSiteMap();
        }

        /// <summary>
        /// Retrieves a site map.
        /// </summary>
        [Test]
        public void TestGetSiteMap()
        {
            IDictionary sitemap = testObject.GetSiteMap();
            INodeMatcher main = new RegexNodeMatcher("Main", "/", "/");
            INodeMatcher child1 = new RegexNodeMatcher("Child1", "/child1", "/child1");
            INodeMatcher childFolder = new RegexNodeMatcher("ChildFolder", "/childFolder/", "/childFolder/.*");
            INodeMatcher child3 = new RegexNodeMatcher("Child3", "/child3", "/child3");
            INodeMatcher child4 = new RegexNodeMatcher("Child4", "/child4", "/child4");
            INodeMatcher child2 = new RegexNodeMatcher("Child2", "/child2", "/child2");
            INodeMatcher altRoot = new RegexNodeMatcher("Alternative root", "/other", "/other");

            Assert.AreEqual(7, sitemap.Count, "Wrong object count.");

            IList list = sitemap[main] as IList;
            Assert.AreEqual(2, list.Count, "Two children to Main expected.");
            Assert.IsTrue(list.Contains(child1), "Main -> Child1");
            Assert.IsTrue(list.Contains(child2), "Main -> Child2");

            list = sitemap[child1] as IList;
            Assert.AreEqual(1, list.Count, "One child to Child1 expected.");
            Assert.IsTrue(list.Contains(childFolder), "Child1 -> ChildFolder");

            list = sitemap[childFolder] as IList;
            Assert.AreEqual(2, list.Count, "Two children to childFolder expected.");
            Assert.IsTrue(list.Contains(child3), "ChildFolder -> Child3");
            Assert.IsTrue(list.Contains(child4), "ChildFolder -> Child4");

            list = sitemap[altRoot] as IList;
            Assert.AreEqual(1, list.Count, "One child to alternative root expected.");
            Assert.IsTrue(list.Contains(childFolder), "Alternative root -> ChildFolder");

            foreach (INodeMatcher node in new INodeMatcher[] { child3, child4, child2 })
            {
                list = sitemap[node] as IList;
                Assert.AreEqual(0, list.Count, "Leaf nodes should be empty.");
            }
        }

        /// <summary>
        /// We don't parse by DOM, which makes us sensitive to the exact XML layout. Let's test what happens if we skip
        /// all whitespace.
        /// </summary>
        [Test]
        public void TestCompactXml()
        {
            testObject = new XmlPathDiscovery(
                @"<BreadCrumb><bc_node title=""Main"" url=""/"" pattern=""/""><bc_node title=""Child1"" " +
                @"url=""/child1"" pattern=""/child1""><bc_node title=""ChildFolder"" url=""/childFolder/"" " +
                @"pattern=""/childFolder/.*""><bc_node title=""Child3"" url=""/child3"" pattern=""/child3"">" +
                @"</bc_node><bc_node title=""Child4"" url=""/child4"" pattern=""/child4""></bc_node></bc_node>" +
                @"</bc_node><bc_node title=""Child2"" url=""/child2"" pattern=""/child2""></bc_node></bc_node>" +
                @"<bc_node title=""Alternative root"" url=""/other"" pattern=""/other""><bc_node " +
                @"title=""ChildFolder"" url=""/childFolder/"" pattern=""/childFolder/.*"" /></bc_node></BreadCrumb>",
                false);

            // This data should be semantically identical
            TestGetSiteMap();
        }

        /// <summary>
        /// A leaf node with an object may appear before a node listing its children.
        /// </summary>
        [Test]
        public void TestReversedXml()
        {
            testObject = new XmlPathDiscovery(
                @"<BreadCrumb>
                    <bc_node title=""Alternative root"" url=""/other"" pattern=""/other"">
                        <bc_node title=""ChildFolder"" url=""/childFolder/"" pattern=""/childFolder/.*"" />
                    </bc_node>
                    <bc_node title=""Main"" url=""/"" pattern=""/"">
                        <bc_node title=""Child1"" url=""/child1"" pattern=""/child1"">
                            <bc_node title=""ChildFolder"" url=""/childFolder/"" pattern=""/childFolder/.*"">
                                <bc_node title=""Child3"" url=""/child3"" pattern=""/child3"">
                                </bc_node>
                                <bc_node title=""Child4"" url=""/child4"" pattern=""/child4"">
                                </bc_node>
                            </bc_node>
                        </bc_node>
                        <bc_node title=""Child2"" url=""/child2"" pattern=""/child2"">
                        </bc_node>
                    </bc_node>
                  </BreadCrumb>", false);

            // This data should be semantically identical
            TestGetSiteMap();
        }

        /// <summary>
        /// Only one occurrence of a node may carry children, even if they are identical.
        /// </summary>
        [Test]
        [ExpectedException(typeof(BreadCrumbException))]
        public void TestChildrenTwice()
        {
            testObject = new XmlPathDiscovery(
                @"<BreadCrumb>
                    <bc_node title=""Alternative root"" url=""/other"" pattern=""/other"">
                        <bc_node title=""ChildFolder"" url=""/childFolder/"" pattern=""/childFolder/.*"">
                           <bc_node title=""Child3"" url=""/child3"" pattern=""/child3"">
                           </bc_node>
                           <bc_node title=""Child4"" url=""/child4"" pattern=""/child4"">
                           </bc_node>
                        </bc_node>
                    </bc_node>
                    <bc_node title=""Main"" url=""/"" pattern=""/"">
                        <bc_node title=""Child1"" url=""/child1"" pattern=""/child1"">
                            <bc_node title=""ChildFolder"" url=""/childFolder/"" pattern=""/childFolder/.*"">
                                <bc_node title=""Child3"" url=""/child3"" pattern=""/child3"">
                                </bc_node>
                                <bc_node title=""Child4"" url=""/child4"" pattern=""/child4"">
                                </bc_node>
                            </bc_node>
                        </bc_node>
                        <bc_node title=""Child2"" url=""/child2"" pattern=""/child2"">
                        </bc_node>
                    </bc_node>
                  </BreadCrumb>", false);

            // This data should be semantically identical
            TestGetSiteMap();
        }

        /// <summary>
        /// Two session variables are used to cache the sitemap data. This test just verifies the constant values.
        /// </summary>
        [Test]
        public void TestConstants()
        {
            Assert.AreEqual("TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery.XmlPathDiscovery.siteMap",
                XmlPathDiscovery.CURRENT_SITEMAP);
            Assert.AreEqual("TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery.XmlPathDiscovery.reversedSiteMap",
                XmlPathDiscovery.REVERSED_SITEMAP);
        }

        /// <summary>
        /// Tests against a null node URL. An exception is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestNullNodeUrl()
        {
            testObject.GetPath(new BreadCrumbNode(), new Page());
        }

        /// <summary>
        /// Tests against a null page. An exception is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestNullPage()
        {
            testObject.GetPath(new BreadCrumbNode("title", "url"), null);
        }
    }
}
