/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER
 * Version: 2.0
 */

using System;
using System.IO;
using System.Web.UI;
using NUnit.Framework;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.PathWriter
{
    /// <summary>
    /// This class tests one of the two provided path writers. Note that we test it sub-method by sub-method. Complete
    /// tests of the functionality provided by the abstract base class and the component itself isn't handled here.
    /// </summary>
    /// <remarks>
    /// We're inheriting from the tested class. It's stateless and we need access to the protected methods.
    /// </remarks>
    [TestFixture]
    public class TestOrderedListPathWriter : OrderedListPathWriter
    {
        /// <summary>
        /// Tests writing a single node, with no styles.
        /// </summary>
        [Test]
        public void TestWriteNode()
        {
            using (StringWriter writer = new StringWriter())
            {
                using (HtmlTextWriter htmlWriter = new HtmlTextWriter(writer))
                {
                    WriteNode(htmlWriter, new BreadCrumbNode("Title", "url"), null, null);
                }

                Assert.AreEqual("<li><a href=\"url\">Title</a></li>", writer.ToString(),
                    "HTML string not matching expectations.");
            }
        }

        /// <summary>
        /// Tests writing a single node, with no styles and an empty title string. A predefined Unknown string is used.
        /// </summary>
        [Test]
        public void TestWriteEmptyTitle()
        {
            using (StringWriter writer = new StringWriter())
            {
                using (HtmlTextWriter htmlWriter = new HtmlTextWriter(writer))
                {
                    WriteNode(htmlWriter, new BreadCrumbNode(" ", "url"), null, null);
                }

                Assert.AreEqual("<li><a href=\"url\">Unknown</a></li>", writer.ToString(),
                    "HTML string not matching expectations.");
            }
        }

        /// <summary>
        /// Tests writing a single node, with no styles and a null title string. A predefined Unknown string is used.
        /// </summary>
        [Test]
        public void TestWriteNullTitle()
        {
            using (StringWriter writer = new StringWriter())
            {
                using (HtmlTextWriter htmlWriter = new HtmlTextWriter(writer))
                {
                    WriteNode(htmlWriter, new BreadCrumbNode(null, "url"), null, null);
                }

                Assert.AreEqual("<li><a href=\"url\">Unknown</a></li>", writer.ToString(),
                    "HTML string not matching expectations.");
            }
        }

        /// <summary>
        /// Tests writing a single node, with just a node style.
        /// </summary>
        [Test]
        public void TestWriteNodeStyle()
        {
            using (StringWriter writer = new StringWriter())
            {
                using (HtmlTextWriter htmlWriter = new HtmlTextWriter(writer))
                {
                    WriteNode(htmlWriter, new BreadCrumbNode("Title", "url"), "node", null);
                }

                Assert.AreEqual("<li><a href=\"url\" class=\"node\">Title</a></li>", writer.ToString(),
                    "HTML string not matching expectations.");
            }
        }

        /// <summary>
        /// Tests writing a single node, with just a mouse over style.
        /// </summary>
        [Test]
        public void TestWriteMouseStyle()
        {
            using (StringWriter writer = new StringWriter())
            {
                using (HtmlTextWriter htmlWriter = new HtmlTextWriter(writer))
                {
                    WriteNode(htmlWriter, new BreadCrumbNode("Title", "url"), null, "mouse");
                }

                Assert.AreEqual(
                    "<li><a href=\"url\" onmouseover=\"this.className='mouse'\" onmouseout=\"this.className=''\">" +
                    "Title</a></li>",
                    writer.ToString(), "HTML string not matching expectations.");
            }
        }

        /// <summary>
        /// Tests writing a single node, with both style types.
        /// </summary>
        [Test]
        public void TestWriteBothStyles()
        {
            using (StringWriter writer = new StringWriter())
            {
                using (HtmlTextWriter htmlWriter = new HtmlTextWriter(writer))
                {
                    WriteNode(htmlWriter, new BreadCrumbNode("Title", "url"), "node", "mouse");
                }

                Assert.AreEqual(
                    "<li><a href=\"url\" class=\"node\" onmouseover=\"this.className='mouse'\" onmouseout=\"" +
                    "this.className='node'\">Title</a></li>", writer.ToString(),
                    "HTML string not matching expectations.");
            }
        }

        /// <summary>
        /// Tests the path starting method, which creates the list.
        /// </summary>
        [Test]
        public void TestBeginPath()
        {
            using (StringWriter writer = new StringWriter())
            {
                using (HtmlTextWriter htmlWriter = new HtmlTextWriter(writer))
                {
                    BeginPath(htmlWriter);
                }

                Assert.AreEqual("<ol>\r\n", writer.ToString(), "The list should begin.");
            }
        }

        /// <summary>
        /// Tests the path starting and ending methods, which creates and closes the list. The ending method cannot
        /// be tested separately easily, as it closes a previously started tag.
        /// </summary>
        [Test]
        public void TestEndPath()
        {
            using (StringWriter writer = new StringWriter())
            {
                using (HtmlTextWriter htmlWriter = new HtmlTextWriter(writer))
                {
                    BeginPath(htmlWriter);
                    EndPath(htmlWriter);
                }

                Assert.AreEqual("<ol>\r\n\r\n</ol>", writer.ToString(), "The list should begin and end.");
            }
        }

        /// <summary>
        /// Tests separator writing, which is here overridden for NO separator.
        /// </summary>
        [Test]
        public void TestWriteSeparator()
        {
            using (StringWriter writer = new StringWriter())
            {
                using (HtmlTextWriter htmlWriter = new HtmlTextWriter(writer))
                {
                    WriteSeparator(htmlWriter, new BreadCrumbTrailControl());
                }

                Assert.AreEqual("", writer.ToString(), "No separator should be written.");
            }
        }

        /// <summary>
        /// Tests ending a path with a null parameter.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestEndPathNull()
        {
            EndPath(null);
        }

        /// <summary>
        /// Tests beginning a path with a null parameter.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestBeginPathNull()
        {
            BeginPath(null);
        }

        /// <summary>
        /// Tests writing a separator with a null writer. An exception should be thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestWriteSeparatorNullWriter()
        {
            WriteSeparator(null, new BreadCrumbTrailControl());
        }

        /// <summary>
        /// Tests writing a separator with a null control. An exception should be thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestWriteSeparatorNullControl()
        {
            using (StringWriter writer = new StringWriter())
            {
                try
                {
                    using (HtmlTextWriter htmlWriter = new HtmlTextWriter(writer))
                    {
                        WriteSeparator(htmlWriter, null);
                    }
                }
                finally
                {
                    Assert.AreEqual("", writer.ToString(), "Nothing should be written.");
                }
            }
        }

        /// <summary>
        /// Tests writing with a null writer, an exception is thrown-
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestWriteNodeNullWriter()
        {
            WriteNode(null, new BreadCrumbNode("Title", "url"), null, null);
        }

        /// <summary>
        /// Tests writing with a "null" node, that is, a node with a null url. An exception should be thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestWriteNodeNullNode()
        {
            using (StringWriter writer = new StringWriter())
            {
                try
                {
                    using (HtmlTextWriter htmlWriter = new HtmlTextWriter(writer))
                    {
                        WriteNode(htmlWriter, new BreadCrumbNode(), null, null);
                    }
                }
                finally
                {
                    Assert.AreEqual("", writer.ToString(),
                        "Nothing should be wrtten.");
                }
            }
        }
    }
}
